% A Simple Inertial Oscillator to Demonstrate the Runge-Kutta Integrator.
%
% This is a simple integration program to solve the differential equations:
% d y(1)/dt=y(3)
% d y(2)/dt=y(4)
% d y(3)/dt-2 opts(1) y(4) - opts(2) opts(1)^2 y(1)=-opts(3) y(3)
% d y(4)/dt+2 opts(1) y(3) - opts(2) opts(1)^2 y(2)=-opts(3) y(4)
% For the independent variables y(1-4), given parameters (opts(1-3))
% over time (t).
%
% It is the eqtn for an inertially oscillating particle, 
% in a rotating frame with angular frequency opts(1)
% and drag coefficient opts(3).  
% opts(2)=0 turns off the Centrifugal force (i.e., constant geopotential)
% opts(2)=1 turns on the Centrifugal force (i.e., constant geopotential)
% One could consider y(1) to be the x-position
% One could consider y(2) to be the y-position
% One could consider y(3) to be the x-velocity component
% One could consider y(4) to be the y-velocity component
% of the oscillator, t to be time.
% d x/dt=u
% d y/dt=v
% d u/dt-2 Omega v + opts(2) Omega^2 x=-r u
% d v/dt+2 Omega u + opts(2) Omega^2 y=-r v
%
% For shorthand, yp is used to denote dy/dt
%
% There are 4 files in this directory:
% This one, which is used to execute the run
%  rkparm.m, a parameter file for the integration paramters
%  rk5.m, the problem-independent Runge-Kutta time-stepper
%  tendencies.m, the function which calculates the right-hand side
%   of the ODEs above.

% This initializes the Runge-Kutta parameters.
rkparm

% Some parameters about saving results
saveint=0.02   % Save values every saveint units of t.
tmax=5        % Final value of tmax.
maxsaves=ceil(tmax/saveint)  % How many saved values to expect?
h=1e-4         % Initial timestep value
steps=0        % counter for the number of steps taken
saves=1        % counter for the number of saved steps

% This is where the initial conditions are set.
t=0;
y(1)=-2; %x
y(2)=0; %y
y(3)=5; %u
y(4)=0; %v

% This is where the parameters are set
opts(1)=-2*pi % frame rotation rate  (units=radians/time)
opts(2)=0     % Centrifugal on/off      (units=none)
opts(3)=0    % drag                   (units=1/time)

% Initialization of arrays to save output for this run.
tsaves=zeros(1,maxsaves);
ysaves=zeros(4,maxsaves);

% Save the initial values!
tsaves(:,1)=t;
ysaves(:,1)=y;

% This begins the clock for efficiency timing.
tic

% Beginning of run loop
while t<tmax
 steps=steps+1;

% This calls the Runge-Kutta timestepper
 [ynew,hnew]=rk5(t,h,y,opts,rk,herr);

% Update time, y, and adaptive timestep.
 t=t+h;
 y=ynew;

% The next timestep, we use either the suggested one from
% rk5, or the distance to the next save interval
 h=min(hnew,eps+saveint-mod(t,saveint));

% Now check to see if we should save this step permanently
 if (mod(t,saveint)<2*eps)
  saves=saves+1;
  tsaves(saves:end)=t;
  ysaves(:,saves)=y;

% A little timer to see how long the whole run will last
% and a simple plotter of the run so far.
  if mod(saves,1)==0 
   [t tmax toc toc*(tmax-t)/tmax]
   figure(1)
   subplot(111)
    quiver(y(1),y(2),y(3)/5,y(4)/5,0,'r','LineWidth',3);axis([-5 5 -5 5]); axis square;
    xlabel('x');ylabel('y');title(['Rotating Frame, Omega=',num2str(opts(1))])
    hold on
     ct=cos(-opts(1)*t);
     st=sin(-opts(1)*t);
     plot([0 4*ct],[0 4*st],'b','LineWidth',3);text(4*ct,4*st,'x_i')
     plot([0 -4*st],[0 4*ct],'g','LineWidth',3);text(-4*st,4*ct,'y_i')
     plot(ysaves(1,:),ysaves(2,:),'.')
    hold off

    ctsaves=cos(opts(1)*tsaves);
    stsaves=sin(opts(1)*tsaves);
   
   figure(2)
   subplot(411)
    plot(tsaves,ysaves(2,:),tsaves,ysaves(1,:).*ctsaves-ysaves(2,:).*stsaves)
    title('Blue Variables in Rotating Frame; Green in Inertial')
    xlabel('time');ylabel('x-position');
   subplot(412)
    plot(tsaves,ysaves(2,:),tsaves,ysaves(1,:).*stsaves+ysaves(2,:).*ctsaves)
    xlabel('time');ylabel('y-position');
   subplot(413)
    plot(tsaves,ysaves(3,:))
    xlabel('time');ylabel('x-velocity');
   subplot(414)
    plot(tsaves,ysaves(4,:))
    xlabel('time');ylabel('y-velocity');

   figure(3)
   subplot(111)
    ct=cos(opts(1)*t);
    st=sin(opts(1)*t);

    plot([0 4*ct],[0 4*st],'b','LineWidth',3);text(4*ct,4*st,'x_r')

    xlabel('x');ylabel('y');title(['Inertial Frame, Omega=',num2str(opts(1))])
    hold on
     plot([0 -4*st],[0 4*ct],'g','LineWidth',3);text(-4*st,4*ct,'y_r')
     plot(ysaves(1,:).*ctsaves-ysaves(2,:).*stsaves,ysaves(1,:).*stsaves+ysaves(2,:).*ctsaves,'.')

     diffx=ysaves(1,saves)*ctsaves(saves)-ysaves(2,saves)*stsaves(saves)-(ysaves(1,saves-1)*ctsaves(saves-1)-ysaves(2,saves-1)*stsaves(saves-1));
     diffy=ysaves(1,saves)*stsaves(saves)+ysaves(2,saves)*ctsaves(saves)-(ysaves(1,saves-1)*stsaves(saves-1)+ysaves(2,saves-1)*ctsaves(saves-1));
     difft=(tsaves(saves)-tsaves(saves-1));

quiver(y(1)*ct-y(2)*st,y(1)*st+y(2)*ct,diffx/difft/5,diffy/difft/5,0,'r','LineWidth',3);axis([-5 5 -5 5]); axis square;
    hold off

  figure(2)
   subplot(411)
    hold on


    hold off
   drawnow
  end
 end
end


